#!/usr/bin/env python
# -*- coding: utf-8; -*-

# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

from typing import Dict
import ads
from ads.common.auth import create_signer, AuthContext
from ads.common.oci_client import OCIClientFactory
from ads.opctl.backend.base import Backend
from ads.model.deployment import ModelDeployment

from oci.data_science.models import ModelDeployment as OCIModelDeployment


class ModelDeploymentBackend(Backend):
    def __init__(self, config: Dict) -> None:
        """
        Initialize a ModelDeployment object given config dictionary.

        Parameters
        ----------
        config: dict
            dictionary of configurations
        """
        self.config = config
        self.oci_auth = create_signer(
            config["execution"].get("auth"),
            config["execution"].get("oci_config", None),
            config["execution"].get("oci_profile", None),
        )
        self.auth_type = config["execution"].get("auth")
        self.profile = config["execution"].get("oci_profile", None)
        self.client = OCIClientFactory(**self.oci_auth).data_science

    def apply(self) -> None:
        """
        Deploy model deployment from YAML.
        """
        wait_for_completion = self.config["execution"].get("wait_for_completion")
        max_wait_time = self.config["execution"].get("max_wait_time")
        poll_interval = self.config["execution"].get("poll_interval")
        with AuthContext():
            ads.set_auth(auth=self.auth_type, profile=self.profile)
            model_deployment = ModelDeployment.from_dict(self.config)
            model_deployment.deploy(
                wait_for_completion=wait_for_completion,
                max_wait_time=max_wait_time,
                poll_interval=poll_interval,
            )
            print("Model Deployment id: ", model_deployment.model_deployment_id)

    def delete(self) -> None:
        """
        Delete model deployment from OCID.
        """
        model_deployment_id = self.config["execution"].get("run_id")
        wait_for_completion = self.config["execution"].get("wait_for_completion")
        max_wait_time = self.config["execution"].get("max_wait_time")
        poll_interval = self.config["execution"].get("poll_interval")
        with AuthContext():
            ads.set_auth(auth=self.auth_type, profile=self.profile)
            model_deployment = ModelDeployment.from_id(model_deployment_id)
            if model_deployment.lifecycle_state in [
                OCIModelDeployment.LIFECYCLE_STATE_DELETED
                or OCIModelDeployment.LIFECYCLE_STATE_DELETING
            ]:
                print(
                    f"Model deployment {model_deployment.model_deployment_id} is either deleted or being deleted."
                )
                return
            if model_deployment.lifecycle_state not in [
                OCIModelDeployment.LIFECYCLE_STATE_ACTIVE,
                OCIModelDeployment.LIFECYCLE_STATE_FAILED,
                OCIModelDeployment.LIFECYCLE_STATE_INACTIVE,
            ]:
                raise Exception(
                    f"Can't delete model deployment {model_deployment.model_deployment_id} when it's in {model_deployment.lifecycle_state} state."
                )
            model_deployment.delete(
                wait_for_completion=wait_for_completion,
                max_wait_time=max_wait_time,
                poll_interval=poll_interval,
            )
            print(
                f"Model Deployment {model_deployment.model_deployment_id} has been deleted."
            )

    def activate(self) -> None:
        """
        Activate model deployment from OCID.
        """
        model_deployment_id = self.config["execution"].get("run_id")
        wait_for_completion = self.config["execution"].get("wait_for_completion")
        max_wait_time = self.config["execution"].get("max_wait_time")
        poll_interval = self.config["execution"].get("poll_interval")
        with AuthContext():
            ads.set_auth(auth=self.auth_type, profile=self.profile)
            model_deployment = ModelDeployment.from_id(model_deployment_id)
            if (
                model_deployment.lifecycle_state
                == OCIModelDeployment.LIFECYCLE_STATE_ACTIVE
            ):
                print(
                    f"Model deployment {model_deployment.model_deployment_id} is already active."
                )
                return

            if (
                model_deployment.lifecycle_state
                == OCIModelDeployment.LIFECYCLE_STATE_INACTIVE
            ):
                model_deployment.activate(
                    wait_for_completion=wait_for_completion,
                    max_wait_time=max_wait_time,
                    poll_interval=poll_interval,
                )
                print(
                    f"Model Deployment {model_deployment.model_deployment_id} has been activated."
                )
            else:
                raise Exception(
                    f"Can't activate model deployment {model_deployment.model_deployment_id} when it's in {model_deployment.lifecycle_state} state."
                )

    def deactivate(self) -> None:
        """
        Deactivate model deployment from OCID.
        """
        model_deployment_id = self.config["execution"].get("run_id")
        wait_for_completion = self.config["execution"].get("wait_for_completion")
        max_wait_time = self.config["execution"].get("max_wait_time")
        poll_interval = self.config["execution"].get("poll_interval")
        with AuthContext():
            ads.set_auth(auth=self.auth_type, profile=self.profile)
            model_deployment = ModelDeployment.from_id(model_deployment_id)
            if (
                model_deployment.lifecycle_state
                == OCIModelDeployment.LIFECYCLE_STATE_INACTIVE
            ):
                print(
                    f"Model deployment {model_deployment.model_deployment_id} is already inactive."
                )
                return

            if (
                model_deployment.lifecycle_state
                == OCIModelDeployment.LIFECYCLE_STATE_ACTIVE
            ):
                model_deployment.deactivate(
                    wait_for_completion=wait_for_completion,
                    max_wait_time=max_wait_time,
                    poll_interval=poll_interval,
                )
                print(
                    f"Model Deployment {model_deployment.model_deployment_id} has been deactivated."
                )
            else:
                raise Exception(
                    f"Can't deactivate model deployment {model_deployment.model_deployment_id} when it's in {model_deployment.lifecycle_state} state."
                )

    def watch(self) -> None:
        """
        Watch Model Deployment from OCID.
        """
        model_deployment_id = self.config["execution"].get("run_id")
        log_type = self.config["execution"].get("log_type")
        interval = self.config["execution"].get("interval")
        log_filter = self.config["execution"].get("log_filter")
        with AuthContext():
            ads.set_auth(auth=self.auth_type, profile=self.profile)
            model_deployment = ModelDeployment.from_id(model_deployment_id)
            model_deployment.watch(
                log_type=log_type, interval=interval, log_filter=log_filter
            )



#!/usr/bin/env python
# -*- coding: utf-8; -*-

# Copyright (c) 2022 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

from ads.opctl.diagnostics.check_requirements import RequirementChecker
from ads.opctl.diagnostics.check_distributed_job_requirements import (
    PortsChecker,
    Default,
)
import argparse

RequirementChecker.register_provider("cluster-ports-check", PortsChecker)
RequirementChecker.register_provider("cluster-generic", Default)

parser = argparse.ArgumentParser(description="Run diagnostics to check your setup")
parser.add_argument(
    "-t",
    "--type",
    help="What type of diagnostics to run. Eg. `distributed` to run diagnostics for distributed training setup",
)

args = parser.parse_args()
RequirementChecker().verify_requirements(args.type)



#!/usr/bin/env python
# -*- coding: utf-8 -*--

# Copyright (c) 2022 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

# The ADSString was moved to the feature_type folder. This INI file created for the back compatability
from ads.feature_engineering.feature_type.adsstring.oci_language import OCILanguage



#!/usr/bin/env python
# -*- coding: utf-8 -*--

# Copyright (c) 2020, 2022 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

from __future__ import print_function, absolute_import
import warnings

warnings.warn(
    (
        "The `ads.catalog.summary` is deprecated in `oracle-ads 2.6.9` and will be removed in `oracle-ads 3.0`."
    ),
    DeprecationWarning,
    stacklevel=2,
)
import abc
import ads.common.utils as utils
from oci.util import to_dict
from pandas import DataFrame
import pandas as pd
from abc import ABCMeta
from ads.common.decorator.runtime_dependency import (
    runtime_dependency,
    OptionalDependency,
)


class SummaryList(list, metaclass=ABCMeta):
    def __init__(self, entity_list, datetime_format=utils.date_format):
        if isinstance(entity_list, filter):
            entity_list = list(entity_list)
        super(SummaryList, self).__init__(entity_list)

        df = DataFrame.from_records([to_dict(entity) for entity in entity_list])
        # edge case handling: handle edge case where dataframe is empty
        if not df.empty:
            # handle collision protection
            nuniques = df["id"].nunique()
            minLen = -6
            shortened_ocids = df["id"].str[minLen:]
            while shortened_ocids.nunique() != nuniques:
                minLen -= 1
                shortened_ocids = df["id"].str[minLen:]
            df.index = shortened_ocids
            self.short_id_index = df["id"].to_dict()
            ordered_columns = [
                "display_name",
                "description",
                "time_created",
                "lifecycle_state",
                "user_name",
                "compartment_id",
                "project_id",
            ]
            remove_list = ["created_by"]

            ordered_columns.extend(
                [column for column in df.columns if column not in ordered_columns]
            )

            for column in ordered_columns:
                if column not in df.columns:
                    ordered_columns.remove(column)
            for column in remove_list:
                ordered_columns.remove(column)
            self.df = df[ordered_columns].drop(columns="id")
            self.df["compartment_id"] = "..." + self.df["compartment_id"].str[-6:]
            if "project_id" in ordered_columns:
                self.df["project_id"] = "..." + self.df["project_id"].str[-6:]
            if "model_version_set_id" in ordered_columns:
                self.df["model_version_set_id"] = (
                    "..." + self.df["model_version_set_id"].str[-6:]
                )
            self.df["time_created"] = pd.to_datetime(
                self.df["time_created"]
            ).dt.strftime(datetime_format)
            self.datetime_format = datetime_format
        else:
            self.df = df

    @abc.abstractmethod
    def sort_by(self, columns, reverse=False):  # pragma: no cover
        """
        Abstract method for sorting, implemented by the derived class
        """
        pass

    @abc.abstractmethod
    def filter(self, selection, instance=None):  # pragma: no cover
        """
        Abstract method for filtering, implemented by the derived class
        """
        pass

    @abc.abstractmethod
    def __add__(self, rhs):  # pragma: no cover
        pass

    def to_dataframe(self, datetime_format=None):

        """
        Returns the model catalog summary as a pandas dataframe

        Parameters
        ----------
        datatime_format: date_format
          A datetime format, like utils.date_format. Defaults to none.

        Returns
        -------
        Dataframe: The pandas DataFrame repersentation of the model catalog summary
        """

        df = self.df.copy()
        if datetime_format is not None:
            df["time_created"] = pd.to_datetime(df["time_created"]).dt.strftime(
                datetime_format
            )
        return df

    @runtime_dependency(module="IPython", install_from=OptionalDependency.NOTEBOOK)
    def show_in_notebook(self, datetime_format=None):

        """
        Displays the model catalog summary in a Jupyter Notebook cell

        Parameters
        ----------
        date_format: like utils.date_format. Defaults to none.

        Returns
        -------
        None
        """
        from IPython.core.display import display

        display(
            self.to_dataframe(datetime_format=datetime_format).style.applymap(
                self._color_lifecycle_state, subset=["lifecycle_state"]
            )
        )

    def _repr_html_(self):
        return self.df.style.applymap(
            self._color_lifecycle_state, subset=["lifecycle_state"]
        ).render()

    def _sort_by(self, cols, reverse=False):
        return sorted(
            self,
            key=lambda x: (
                [
                    getattr(x, col).lower()
                    if isinstance(getattr(x, col), str)
                    else getattr(x, col)
                    for col in cols
                ]
            ),
            reverse=reverse,
        )

    def _color_lifecycle_state(self, lifecycle_state):
        """
        Takes a scalar and returns a string with
        the css property
        """
        if lifecycle_state == "INACTIVE":
            color = "grey"
        elif lifecycle_state == "ACTIVE":
            color = "green"
        elif lifecycle_state == "DELETING":
            color = "black"
        elif lifecycle_state == "CREATING":
            color = "blue"
        elif lifecycle_state == "DELETED":
            color = "black"
        elif lifecycle_state == "FAILED":
            color = "red"
        return "color: %s" % color



#!/usr/bin/env python
# -*- coding: utf-8 -*--

# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

import json
import os
from functools import lru_cache
from typing import Dict, Tuple
import logging

import fsspec
import requests
import yaml
from cerberus import DocumentError, Validator

from ads.common.object_storage_details import ObjectStorageDetails
from ads.common.utils import PAR_LINK

MODEL_PROVENANCE_SCHEMA_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "schemas",
    "model_provenance_schema.yaml",
)
INFERENCE_ENV_SCHEMA_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "schemas",
    "inference_env_info_schema.yaml",
)
TRAINING_ENV_SCHEMA_PATH = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    "schemas",
    "training_env_info_schema.yaml",
)
SERVICE_PACKS = "service_packs"


class SchemaValidator:
    """
    Base Schema Validator which validate yaml file.
    """

    def __init__(self, schema_file_path: str) -> "SchemaValidator":
        """Initiate a SchemaValidator instance.

        Parameters
        ----------
        schema_file_path: (str)
            schema file path. The schema is used to validate the yaml file.

        Returns
        -------
        SchemaValidator
            A SchemaValidator instance.
        """
        self.schema_file_path = schema_file_path
        self._schema_validator = self._load_schema_validator()

    def validate(self, document: Dict) -> bool:
        """Validate the schema.

        Parameters
        ----------
        document: (Dict)
            yaml file content to validate.

        Raises
        ------
        DocumentError: Raised when the validation schema is missing, has the wrong format or contains errors.

        Returns
        -------
        bool
            validation result.
        """
        v = Validator()
        res = v.validate(document, self._schema_validator)
        if not res:
            raise DocumentError(v.errors)
        return res

    def _load_schema_validator(self):
        """load the schema validator to validate the schema.

        Returns
        -------
        dict
            Schema validator.
        """
        with open(self.schema_file_path, encoding="utf-8") as schema_file:
            ext = os.path.splitext(self.schema_file_path)[-1].lower()
            if ext in [".yaml", ".yml"]:
                schema_validator = yaml.load(schema_file, yaml.FullLoader)
            elif ext in [".json"]:
                schema_validator = json.load(schema_file)
            else:
                raise NotImplementedError(f"{ext} format schema is not supported.")
        return schema_validator


def _get_index_json_through_bucket(
    namespace: str, bucketname: str, auth: dict = None
) -> list:
    """get the index json from the object storage.

    Parameters
    ----------
    namespace: str
        The Object Storage namespace.
    bucketname: str
        The Object Storage bucketname.
    auth: (Dict, optional). Defaults to None.
        The default authetication is set using `ads.set_auth` API. If you need to override the
        default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
        authentication signer and kwargs required to instantiate IdentityClient object.

    Returns
    -------
    list: A list of dictionary which contains service packs information.
    """
    auth = auth or {}
    service_pack_list = []
    try:
        uri = f"oci://{bucketname}@{namespace}/service_pack/index.json"
        with fsspec.open(uri, "r", **auth) as f:
            service_packs = json.loads(f.read())
        service_pack_list = service_packs.get(SERVICE_PACKS)
    except Exception as e:
        logging.warn(e)
        logging.warn(
            "Failed to retrieve the full conda pack path from slug. Pass conda pack path 'oci://<bucketname>@<namespace>/<path_to_conda>' instead of slug."
        )
    return service_pack_list


def get_service_packs(
    namespace: str, bucketname: str, auth: dict = None
) -> Tuple[Dict, Dict]:
    """Get the service pack path mapping and service pack slug mapping.
    Note: deprecated packs are also included.

    Parameters
    ----------
    namespace: str
        namespace of the service pack.
    bucketname: str
        bucketname of the service pack.
    auth: (Dict, optional). Defaults to None.
        The default authetication is set using `ads.set_auth` API. If you need to override the
        default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
        authentication signer and kwargs required to instantiate IdentityClient object.

    Returns
    -------
    (Dict, Dict)
        Service pack path mapping(service pack path -> (slug, python version))
        and the service pack slug mapping(service pack slug -> (pack path, python version)).
    """
    service_pack_path_mapping = {}
    service_pack_slug_mapping = {}
    try:
        response = requests.request("GET", PAR_LINK)

        # if there is internet
        if response.ok:
            service_pack_list = response.json().get(SERVICE_PACKS)
        # response not good.
        else:
            service_pack_list = _get_index_json_through_bucket(
                namespace=namespace, bucketname=bucketname, auth=auth
            )
    except Exception as e:
        # not internet
        service_pack_list = _get_index_json_through_bucket(
            namespace=namespace, bucketname=bucketname, auth=auth
        )

    for service_pack in service_pack_list:
        # Here we need to replace the namespace and bucketname
        # with the bucket and namespace of the region that
        # user is in. The reason is that the mapping is generated
        # from the index.json file which has static namespace
        # and bucket of prod. however, namespace will change based
        # on the region. also, dev has different bucketname.
        pack_path = ObjectStorageDetails(
            bucket=bucketname,
            namespace=namespace,
            filepath=ObjectStorageDetails.from_path(
                service_pack.get("pack_path")
            ).filepath,
        ).path
        service_pack_path_mapping[pack_path] = (
            service_pack.get("slug"),
            service_pack.get("python"),
        )
        service_pack_slug_mapping[service_pack.get("slug")] = (
            pack_path,
            service_pack.get("python"),
        )
    return service_pack_path_mapping, service_pack_slug_mapping
